unit FreeOTFESettings;
// Description: 
// By Sarah Dean
// Email: sdean12@sdean12.org
// WWW:   http://www.FreeOTFE.org/
//
// -----------------------------------------------------------------------------
//


interface

uses
  Classes, // Required for TShortCut
  SDUMRUList;

type
  TDragDropFileType = (ftPrompt, ftFreeOTFE, ftLinux);

  TSettingsSaveLocation = (slNone, slExeDir, slProfile);

const
  DragDropFileTypeTitle: array [TDragDropFileType] of string = (
                                  '<prompt user>',
                                  'FreeOTFE',
                                  'Linux'
                                 );

  SettingsSaveLocationTitle: array [TSettingsSaveLocation] of string = (
                                  '<Do not save settings>',
                                  'FreeOTFE executable dir',
                                  'User profile'
                                 );

  SettingsSaveLocationSearchOrder: array [TSettingsSaveLocation] of TSettingsSaveLocation = (
                                  slExeDir,
                                  slProfile,
                                  slNone
                                 );

type
  TFreeOTFESettings = class
  protected
    function IdentifyWhereSettingsStored(): TSettingsSaveLocation;
  public
    OptSettingsVersion: integer;
    OptSaveSettings: TSettingsSaveLocation;

    // General...
    OptExploreAfterMount: boolean;
    OptAllowMultipleInstances: boolean;
    OptDisplayToolbar: boolean;
    OptDisplayStatusbar: boolean;
    OptAutoStartPortable: boolean;
    OptAdvancedMountDlg: boolean;
    OptDragDropFileType: TDragDropFileType;
    OptRevertVolTimestamps: boolean;
    OptDefaultDriveLetter: char;

    // System tray icon...
    OptSystemTrayIconDisplay: boolean;
    OptSystemTrayIconMinTo: boolean;
    OptSystemTrayIconCloseTo: boolean;

    // Hotkey...
    OptHKeyEnableDismount: boolean;
    OptHKeyKeyDismount: TShortCut;
    OptHKeyEnableDismountEmerg: boolean;
    OptHKeyKeyDismountEmerg: TShortCut;

    // Autorun...
    OptPostMountExe: string;
    OptPreDismountExe: string;
    OptPostDismountExe: string;
    OptPrePostExeWarn: boolean;

    // PKCS11...
    OptPKCS11Enable: boolean;
    OptPKCS11Library: string;
    OptPKCS11AutoMount: boolean;
    OptPKCS11AutoMountVolume: string;
    OptPKCS11AutoDismount: boolean;

    // MRU list...
    OptMRUList: TSDUMRUList;

    constructor Create();
    destructor Destroy(); override;
    procedure Load();
    function Save(): boolean;

    function DestroySettingsFile(loc: TSettingsSaveLocation): boolean;
    function GetSettingsFilename(loc: TSettingsSaveLocation): string;

  end;

var
  // Global variable
  Settings: TFreeOTFESettings;

implementation

uses
  INIFiles,
  Windows,  // Required to get rid of compiler hint re DeleteFile
  SysUtils,  // Required for ChangeFileExt, DeleteFile
  Dialogs,
  SDUGeneral,
  SDUDialogs,
  Menus,  // Required for ShortCutToText and TextToShortCut
  ShlObj;  // Required for CSIDL_PERSONAL

const
  SETTINGS_V1 = 1;
  SETTINGS_V2 = 2;

  // -- General section --
  SECTION_GENERAL = 'General';
    OPT_SETTINGSVERSION                          = 'SettingsVersion';
      DFLT_OPT_SETTINGSVERSION                      = SETTINGS_V2;
    OPT_SAVESETTINGS                             = 'SaveSettings';
      DFLT_OPT_SAVESETTINGS                         = slProfile;
    OPT_EXPLOREAFTERMOUNT                        = 'ExploreAfterMount';
      DFLT_OPT_EXPLOREAFTERMOUNT                    = FALSE;
    OPT_ALLOWMULTIPLEINSTANCES                   = 'AllowMultipleInstances';
      DFLT_OPT_ALLOWMULTIPLEINSTANCES               = FALSE;
    OPT_DISPLAYTOOLBAR                           = 'DisplayToolbar';
      DFLT_OPT_DISPLAYTOOLBAR                       = TRUE;
    OPT_DISPLAYSTATUSBAR                         = 'DisplayStatusbar';
      DFLT_OPT_DISPLAYSTATUSBAR                     = TRUE;
    OPT_AUTOSTARTPORTABLE                         = 'AutoStartPortable';
      DFLT_OPT_AUTOSTARTPORTABLE                    = FALSE;
    OPT_ADVANCEDMOUNTDLG                         = 'AdvancedMountDlg';
      DFLT_OPT_ADVANCEDMOUNTDLG                     = FALSE;
    OPT_DRAGDROP                                 = 'DragDropFileType';
      DFLT_OPT_DRAGDROP                             = ord(ftFreeOTFE);
    OPT_REVERTVOLTIMESTAMPS                      = 'RevertVolTimestamps';
      DFLT_OPT_REVERTVOLTIMESTAMPS                  = TRUE;
    OPT_DEFAULTDRIVELETTER                       = 'DefaultDriveLetter';
      DFLT_OPT_DEFAULTDRIVELETTER                   = '#';

  // -- System tray icon section --
  SECTION_SYSTEMTRAYICON = 'SystemTrayIcon';
    OPT_SYSTEMTRAYICONDISPLAY                    = 'Display';
      DFLT_OPT_SYSTEMTRAYICONDISPLAY                = FALSE;
    OPT_SYSTEMTRAYICONMINTO                      = 'MinTo';
      DFLT_OPT_SYSTEMTRAYICONMINTO                  = FALSE;
    OPT_SYSTEMTRAYICONCLOSETO                    = 'CloseTo';
      DFLT_OPT_SYSTEMTRAYICONCLOSETO                = FALSE;

  // -- Hotkey section --
  SECTION_HOTKEY = 'Hotkeys';
    OPT_HOTKEYENABLEDISMOUNT                     = 'EnableDismount';
      DFLT_OPT_HOTKEYENABLEDISMOUNT                 = FALSE;
    OPT_HOTKEYKEYDISMOUNT                        = 'KeyDismount';
      DFLT_OPT_HOTKEYKEYDISMOUNT                    = 'Shift+Ctrl+D';
    OPT_HOTKEYENABLEDISMOUNTEMERG                = 'EnableDismountEmerg';
      DFLT_OPT_HOTKEYENABLEDISMOUNTEMERG            = FALSE;
    OPT_HOTKEYKEYDISMOUNTEMERG                   = 'KeyDismountEmerg';
      DFLT_OPT_HOTKEYKEYDISMOUNTEMERG               = 'Shift+Ctrl+Alt+D';

  // -- Autorun section --
  SECTION_AUTORUN = 'Autorun';
    OPT_POSTMOUNTEXE                             = 'PostMountExe';
      DFLT_OPT_POSTMOUNTEXE                         = '';
    OPT_PREDISMOUNTEXE                           = 'PreDismountExe';
      DFLT_OPT_PREDISMOUNTEXE                       = '';
    OPT_POSTDISMOUNTEXE                          = 'PostDismountExe';
      DFLT_OPT_POSTDISMOUNTEXE                      = '';
    OPT_PREPOSTEXEWARN                           = 'PrePostExeWarn';
      DFLT_OPT_PREPOSTEXEWARN                       = FALSE;

  // -- PKCS#11 section --
  SECTION_PKCS11 = 'PKCS11';
    OPT_PKCS11ENABLE                             = 'Enabled';
      DFLT_OPT_PKCS11ENABLE                        = FALSE;
    OPT_PKCS11LIBRARY                            = 'Library';
      DFLT_OPT_PKCS11LIBRARY                       = '';
    OPT_PKCS11AUTOMOUNT                          = 'AutoMount';
      DFLT_OPT_PKCS11AUTOMOUNT                     = FALSE;
    OPT_PKCS11AUTOMOUNTVOLUME                    = 'AutoMountVolume';
      DFLT_OPT_PKCS11AUTOMOUNTVOLUME               = '';
    OPT_PKCS11AUTODISMOUNT                       = 'AutoDismount';
      DFLT_OPT_PKCS11AUTODISMOUNT                  = FALSE;

  // -- MRU list section --
  SECTION_MRULIST = 'MRUList';
    // (No name for this one)
      DFLT_OPT_MRUMAXITEMS                       = 0;

constructor TFreeOTFESettings.Create();
begin
  inherited;

  OptMRUList:= TSDUMRUList.Create();

  Load();

end;

destructor TFreeOTFESettings.Destroy();
begin
  OptMRUList.Free();
  
  inherited;
end;

procedure TFreeOTFESettings.Load();
var
  iniFile: TINIFile;
  useDefaultDriveLetter: string;
begin
  OptSaveSettings:= IdentifyWhereSettingsStored();
  iniFile := TINIFile.Create(GetSettingsFilename(OptSaveSettings));
  try
    OptSettingsVersion := iniFile.ReadInteger(SECTION_GENERAL, OPT_SETTINGSVERSION, SETTINGS_V1);

    OptExploreAfterMount       := iniFile.ReadBool(SECTION_GENERAL,   OPT_EXPLOREAFTERMOUNT,      DFLT_OPT_EXPLOREAFTERMOUNT);
    OptAllowMultipleInstances  := iniFile.ReadBool(SECTION_GENERAL,   OPT_ALLOWMULTIPLEINSTANCES, DFLT_OPT_ALLOWMULTIPLEINSTANCES);
    OptDisplayToolbar          := iniFile.ReadBool(SECTION_GENERAL,   OPT_DISPLAYTOOLBAR,         DFLT_OPT_DISPLAYTOOLBAR);
    OptDisplayStatusbar        := iniFile.ReadBool(SECTION_GENERAL,   OPT_DISPLAYSTATUSBAR,       DFLT_OPT_DISPLAYSTATUSBAR);
    OptAutoStartPortable       := iniFile.ReadBool(SECTION_GENERAL,   OPT_AUTOSTARTPORTABLE,      DFLT_OPT_AUTOSTARTPORTABLE);
    OptAdvancedMountDlg        := iniFile.ReadBool(SECTION_GENERAL,   OPT_ADVANCEDMOUNTDLG,       DFLT_OPT_ADVANCEDMOUNTDLG);
    OptDragDropFileType        := TDragDropFileType(iniFile.ReadInteger(SECTION_GENERAL, OPT_DRAGDROP, DFLT_OPT_DRAGDROP));
    OptRevertVolTimestamps     := iniFile.ReadBool(SECTION_GENERAL,   OPT_REVERTVOLTIMESTAMPS,    DFLT_OPT_REVERTVOLTIMESTAMPS);
    useDefaultDriveLetter      := iniFile.ReadString(SECTION_GENERAL, OPT_DEFAULTDRIVELETTER,     DFLT_OPT_DEFAULTDRIVELETTER);
    // #0 written as "#"
    OptDefaultDriveLetter := useDefaultDriveLetter[1];
    if (OptDefaultDriveLetter = '#') then
      begin
      OptDefaultDriveLetter := #0;
      end;

    OptSystemTrayIconDisplay   := iniFile.ReadBool(SECTION_SYSTEMTRAYICON, OPT_SYSTEMTRAYICONDISPLAY, DFLT_OPT_SYSTEMTRAYICONDISPLAY);
    OptSystemTrayIconMinTo     := iniFile.ReadBool(SECTION_SYSTEMTRAYICON, OPT_SYSTEMTRAYICONMINTO,   DFLT_OPT_SYSTEMTRAYICONMINTO);
    OptSystemTrayIconCloseTo   := iniFile.ReadBool(SECTION_SYSTEMTRAYICON, OPT_SYSTEMTRAYICONCLOSETO, DFLT_OPT_SYSTEMTRAYICONCLOSETO);

    OptHKeyEnableDismount      := iniFile.ReadBool(SECTION_HOTKEY,   OPT_HOTKEYENABLEDISMOUNT,      DFLT_OPT_HOTKEYENABLEDISMOUNT);
    OptHKeyKeyDismount         := TextToShortCut(iniFile.ReadString(SECTION_HOTKEY, OPT_HOTKEYKEYDISMOUNT,         DFLT_OPT_HOTKEYKEYDISMOUNT));
    OptHKeyEnableDismountEmerg := iniFile.ReadBool(SECTION_HOTKEY,   OPT_HOTKEYENABLEDISMOUNTEMERG, DFLT_OPT_HOTKEYENABLEDISMOUNTEMERG);
    OptHKeyKeyDismountEmerg    := TextToShortCut(iniFile.ReadString(SECTION_HOTKEY, OPT_HOTKEYKEYDISMOUNTEMERG,    DFLT_OPT_HOTKEYKEYDISMOUNTEMERG));

    OptPostMountExe            := iniFile.ReadString(SECTION_AUTORUN,      OPT_POSTMOUNTEXE,          DFLT_OPT_POSTMOUNTEXE);
    OptPreDismountExe          := iniFile.ReadString(SECTION_AUTORUN,      OPT_PREDISMOUNTEXE,        DFLT_OPT_PREDISMOUNTEXE);
    OptPostDismountExe         := iniFile.ReadString(SECTION_AUTORUN,      OPT_POSTDISMOUNTEXE,       DFLT_OPT_POSTDISMOUNTEXE);
    OptPrePostExeWarn          := iniFile.ReadBool(SECTION_AUTORUN,        OPT_PREPOSTEXEWARN,        DFLT_OPT_PREPOSTEXEWARN);

    OptPKCS11Enable            := iniFile.ReadBool(SECTION_PKCS11,         OPT_PKCS11ENABLE,          DFLT_OPT_PKCS11ENABLE);
    OptPKCS11Library           := iniFile.ReadString(SECTION_PKCS11,       OPT_PKCS11LIBRARY,         DFLT_OPT_PKCS11LIBRARY);
    OptPKCS11AutoMount         := iniFile.ReadBool(SECTION_PKCS11,         OPT_PKCS11AUTOMOUNT,       DFLT_OPT_PKCS11AUTOMOUNT);
    OptPKCS11AutoMountVolume   := iniFile.ReadString(SECTION_PKCS11,       OPT_PKCS11AUTOMOUNTVOLUME, DFLT_OPT_PKCS11AUTOMOUNTVOLUME);
    OptPKCS11AutoDismount      := iniFile.ReadBool(SECTION_PKCS11,         OPT_PKCS11AUTODISMOUNT,    DFLT_OPT_PKCS11AUTODISMOUNT);

    OptMRUList.MaxItems := DFLT_OPT_MRUMAXITEMS;
    OptMRUList.Load(iniFile, SECTION_MRULIST);

  finally
    iniFile.Free();
  end;

end;

function TFreeOTFESettings.Save(): boolean;
var
  allOK: boolean;
  iniFile: TINIFile;
  filename: string;
  useDefaultDriveLetter: char;
begin
  allOK := TRUE;
  if (OptSaveSettings <> slNone) then
    begin
    filename := GetSettingsFilename(OptSaveSettings);
    try
      iniFile := TINIFile.Create(filename);
      try
        iniFile.WriteInteger(SECTION_GENERAL,     OPT_SETTINGSVERSION,           SETTINGS_V2);

        iniFile.WriteBool(SECTION_GENERAL,        OPT_EXPLOREAFTERMOUNT,         OptExploreAfterMount);
        iniFile.WriteBool(SECTION_GENERAL,        OPT_ALLOWMULTIPLEINSTANCES,    OptAllowMultipleInstances);
        iniFile.WriteBool(SECTION_GENERAL,        OPT_DISPLAYTOOLBAR,            OptDisplayToolbar);
        iniFile.WriteBool(SECTION_GENERAL,        OPT_DISPLAYSTATUSBAR,          OptDisplayStatusbar);
        iniFile.WriteBool(SECTION_GENERAL,        OPT_AUTOSTARTPORTABLE,         OptAutoStartPortable);
        iniFile.WriteBool(SECTION_GENERAL,        OPT_ADVANCEDMOUNTDLG,          OptAdvancedMountDlg);
        iniFile.WriteInteger(SECTION_GENERAL,     OPT_DRAGDROP,                  ord(OptDragDropFileType));
        iniFile.WriteBool(SECTION_GENERAL,        OPT_REVERTVOLTIMESTAMPS,       OptRevertVolTimestamps);
        // #0 written as "#"
        useDefaultDriveLetter := OptDefaultDriveLetter;
        if (OptDefaultDriveLetter = #0) then
          begin
          useDefaultDriveLetter := '#';
          end;
        iniFile.WriteString(SECTION_GENERAL,      OPT_DEFAULTDRIVELETTER,        useDefaultDriveLetter);

        iniFile.WriteBool(SECTION_SYSTEMTRAYICON, OPT_SYSTEMTRAYICONDISPLAY,     OptSystemTrayIconDisplay);
        iniFile.WriteBool(SECTION_SYSTEMTRAYICON, OPT_SYSTEMTRAYICONMINTO,       OptSystemTrayIconMinTo);
        iniFile.WriteBool(SECTION_SYSTEMTRAYICON, OPT_SYSTEMTRAYICONCLOSETO,     OptSystemTrayIconCloseTo);

        iniFile.WriteString(SECTION_AUTORUN,      OPT_POSTMOUNTEXE,              OptPostMountExe);
        iniFile.WriteString(SECTION_AUTORUN,      OPT_PREDISMOUNTEXE,            OptPreDismountExe);
        iniFile.WriteString(SECTION_AUTORUN,      OPT_POSTDISMOUNTEXE,           OptPostDismountExe);
        iniFile.WriteBool(SECTION_AUTORUN,        OPT_PREPOSTEXEWARN,            OptPrePostExeWarn);

        iniFile.WriteBool(SECTION_PKCS11,         OPT_PKCS11ENABLE,              OptPKCS11Enable);
        iniFile.WriteString(SECTION_PKCS11,       OPT_PKCS11LIBRARY,             OptPKCS11Library);
        iniFile.WriteBool(SECTION_PKCS11,         OPT_PKCS11AUTOMOUNT,           OptPKCS11AutoMount);
        iniFile.WriteString(SECTION_PKCS11,       OPT_PKCS11AUTOMOUNTVOLUME,     OptPKCS11AutoMountVolume);
        iniFile.WriteBool(SECTION_PKCS11,         OPT_PKCS11AUTODISMOUNT,        OptPKCS11AutoDismount);

        OptMRUList.Save(iniFile, SECTION_MRULIST);

      finally
        iniFile.Free();
      end;

    except
      on E: Exception do
        begin
        allOK := FALSE;

        SDUMessageDlg(
                   'Your settings could not be saved.'+SDUCRLF+
                   SDUCRLF+
                   'Please ensure that you have suitable access rights in order to write to:'+SDUCRLF+
                   SDUCRLF+
                   filename,
                   mtError,
                   [mbOK],
                   0
                  );
        end;
    end;

    end;

  Result := allOK;
end;

function TFreeOTFESettings.DestroySettingsFile(loc: TSettingsSaveLocation): boolean;
var
  filename: string;
begin
  filename := GetSettingsFilename(loc);
  Result := DeleteFile(filename);
end;

function TFreeOTFESettings.GetSettingsFilename(loc: TSettingsSaveLocation): string;
var
  iniFilenameOnly: string;
  filenameAndPath: string;
begin
  filenameAndPath := '';

  iniFilenameOnly := ChangeFileExt(ExtractFilename(ParamStr(0)), '.ini');

  if (loc = slNone) then
    begin
    // Do nothing; already set to empty string
    end
  else if (loc = slExeDir) then
    begin
    filenameAndPath := ExtractFilePath(ParamStr(0)) + iniFilenameOnly;
    end
  else if (loc = slProfile) then
    begin
    filenameAndPath := SDUGetSpecialFolderPath(CSIDL_PERSONAL)+
                       '\'+
                       iniFilenameOnly
    end
  else
    begin
    showmessage('Unknown settings location?! Please report seeing this message!');
    end;

  Result := filenameAndPath;
end;

// Identify where user settings are stored
function TFreeOTFESettings.IdentifyWhereSettingsStored(): TSettingsSaveLocation;
var
  sl: TSettingsSaveLocation;
  filename: string;
  retval: TSettingsSaveLocation;
  checkLocation: TSettingsSaveLocation;
begin
  retval := slNone;

  for sl := low(SettingsSaveLocationSearchOrder) to high(SettingsSaveLocationSearchOrder) do
    begin
    checkLocation := SettingsSaveLocationSearchOrder[sl];
    filename := GetSettingsFilename(checkLocation);
    if (
        (filename <> '') and
        FileExists(filename)
       ) then
      begin
      retval := checkLocation;
      break;
      end;

    end;

  Result := retval;
end;


END.


